# et:ts=4
# python24-1.0.tcl
#
# $Id: python24-1.0.tcl 26177 2007-06-15 10:11:22Z jmpp@macports.org $
#
# Copyright (c) 2004 Markus W. Weissman <mww@macports.org>,
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. Neither the name of Apple Computer, Inc. nor the names of its
#    contributors may be used to endorse or promote products derived from
#    this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# arbitrary 'reasonable' defaults
set python.versions {python24 python25 python26}
set python.default_version python26

# Can be removed once MacPorts 1.7.0 is released
if {![info exists frameworks_dir]} {
    set frameworks_dir ${prefix}/Library/Frameworks
}

# Python versions 2.6 and above have different prefix locations (see #16933).
set python24.prefix  ${prefix}
set python25.prefix  ${prefix}
set python26.prefix  ${frameworks_dir}/Python.framework/Versions/2.6

proc python.versions {versions} {
    global python.versions
    set python.versions $versions
}

categories      python
dist_subdir     python

use_configure   no

build.target    build

pre-destroot    {
    xinstall -d -m 755 ${destroot}${prefix}/share/doc/${name}/examples
}

# select a default version if none set
foreach v ${python.versions} {
    if {[variant_isset $v]} {
        set python.version_chosen ${v}
    }
}
if {![info exists python.version_chosen]} {
    variant_set ${python.default_version}
    set python.primary_version ${python.default_version}
} else {
    set python.primary_version python.version_chosen
}

foreach v ${python.versions} {
    # pythonx.y
    set python.dotversion [string range $v 0 end-1].[string index $v end]

    set ${v}.bin        [set ${v}.prefix]/bin/${python.dotversion}
    set ${v}.libdir     [set ${v}.prefix]/lib/${python.dotversion}
    set ${v}.pkgd       [set ${v}.prefix]/lib/${python.dotversion}/site-packages
    set ${v}.include    [set ${v}.prefix]/include/${python.dotversion}
    set ${v}.version    [string range ${v} end-1 end-1].[string range ${v} end end]
    set ${v}.framework  ${frameworks_dir}/Python.framework/Versions/[set ${v}.version]

    # Each python variant gets its own worksrcpath, but they aren't split until after post-patch.
    # Patches applied to all variants can be done in pre-patch, patch, and post-patch.
    # Patches applied to individual variants must be done in pre-build.
    #
    # Each python variant get its own destroot directory.
    # They aren't combined until after post-destroot.
    variant $v "post-patch { copy \${worksrcpath} \${worksrcpath}-${v} }\n  post-destroot { private_python_post_destroot ${v} } "

    # Python versions 2.6 and above have different library locations (see #16933).
    # Python versions 2.6 and above support the --no-user-cfg option (see #16765).
    if { [set ${v}.version] <= 2.5 } {
	set ${v}.lib    [set ${v}.prefix]/lib/lib[set ${v}.version].dylib
	set ${v}.setup  setup.py
    } else {
	set ${v}.lib    [set ${v}.prefix]/Python
	set ${v}.setup  {setup.py --no-user-cfg}
    }

    if {[variant_isset $v]} {
        depends_lib-append      port:${v}
    }
}

build {
    foreach v ${python.versions} {
        if {[variant_isset $v]} {
            set buildcmd "[set ${v}.bin] [set ${v}.setup]"
            system "cd ${worksrcpath}-${v} && ${buildcmd} ${build.target}"
        }
    }
}

destroot {
    # Each python variant has its own destroot.
    # This makes appending version numbers to binary files easier.
    foreach v ${python.versions} {
	if {[variant_isset $v]} {
	    copy ${destroot} ${destroot}-${v}
	}
    }

    foreach v ${python.versions} {
        if {[variant_isset ${v}]} {
	    destroot.destdir    --prefix=[set ${v}.prefix] --root=${destroot}-${v}
            set destrootcmd "[set ${v}.bin] [set ${v}.setup]"

	    system "cd ${worksrcpath}-${v} && ${destrootcmd} ${destroot.target} ${destroot.destdir}"

	    # Avoid conflicts by moving everything out of ${prefix}/[bin, share/man, ...]
	    foreach dir {bin lib/pkgconfig share/pkgconfig} {
		xinstall -d -m 755 ${destroot}-${v}[set ${v}.framework]/${dir}
		foreach file [glob -nocomplain -directory ${destroot}-${v}${prefix}/${dir} *] {
		    move ${file} ${destroot}-${v}[set ${v}.framework]/${dir}/
		}
	    }
	    foreach mani [glob -nocomplain -directory ${destroot}-${v}${prefix}/share/man *] {
		foreach man [glob -nocomplain -directory ${mani} *] {
		    xinstall -d -m 755 ${destroot}-${v}[set ${v}.framework]/share/man/[file tail ${mani}]
		    move ${man} ${destroot}-${v}[set ${v}.framework]/share/man/[file tail ${mani}]/
		}
	    }

	    # Link program files back to ${prefix}/bin with version number appended
	    foreach file [glob -nocomplain -tails -directory ${destroot}-${v}[set ${v}.framework]/bin *] {
		if { [string range ${file} end-2 end] != [set ${v}.version] } {
		    # Don't create link if file ends in version number already
		    ln -s [set ${v}.framework]/bin/${file} ${destroot}${prefix}/bin/${file}-[set ${v}.version]
		}
	    }
	}
    }
}

########################################################################################################################
# Each python variant has its own destroot directory.
# This function combines them.
# Conflicts cause errors
########################################################################################################################
proc private_rename_recur {olddir newdir} {
    foreach fl [glob -nocomplain -tails -directory ${olddir} *] {
	if { [file isdirectory ${olddir}/${fl}] && [file type ${olddir}/${fl}] != "link" } {
	    xinstall -d ${newdir}/${fl}
	    private_rename_recur ${olddir}/${fl} ${newdir}/${fl}
	} else {
	    move ${olddir}/${fl} ${newdir}/${fl}
	}
    }
}

########################################################################################################################
# Called by post-destroot for each variant (v is the python version)
########################################################################################################################
proc private_python_post_destroot {v} {
    global destroot frameworks_dir destroot python.versions prefix
    foreach vv ${python.versions} {
	global ${vv}.version ${vv}.framework
    }

    private_rename_recur ${destroot}-${v} ${destroot}

    # Create links to the 'Current' python version as defined by python_select
    foreach file [glob -nocomplain -tails -directory ${destroot}[set ${v}.framework]/bin *] {
	if { [string range ${file} end-2 end] ne [set ${v}.version] } {
	    # Don't create link if file ends in the version number
	    ln -sf ${frameworks_dir}/Python.framework/Versions/Current/bin/${file} ${destroot}${prefix}/bin/${file} 
	}
    }
    foreach dir {lib/pkgconfig share/pkgconfig} {
	foreach file [glob -nocomplain -tails -directory ${destroot}[set ${v}.framework]/${dir} *] {
	    ln -sf ${frameworks_dir}/Python.framework/Versions/Current/${dir}/${file} ${destroot}${prefix}/${dir}/${file} 
	}
    }
    foreach mani [glob -nocomplain -tails -directory ${destroot}[set ${v}.framework]/share/man *] {
	foreach man [glob -nocomplain -tails -directory ${destroot}[set ${v}.framework]/share/man/${mani} *] {
	    ln -sf ${frameworks_dir}/Python.framework/Versions/Current/share/man/${mani}/${man} ${destroot}${prefix}/share/man/${mani}
	}
    }
}

test {
    foreach v ${python.versions} {
        if {[variant_isset $v]} {
            set destrootcmd "[set ${v}.bin] [set ${v}.setup]"
            system "cd ${worksrcpath}-${v} && ${destrootcmd} test"
        }
    }
}
# Othere possibe test procedures are:
#test {
#    foreach v ${python.versions} {
#        if {[variant_isset $v]} {
#            system "cd ${worksrcpath}-${v} && make check PYTHONPATH=[glob -nocomplain ${worksrcpath}-${v}/build/lib*]"
#        }
#    }
#}